var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { EventEmitter } from 'events';
import { mergeDeepLeft, pick, prop } from 'ramda';
import { join } from 'path';
import { CheckOptions, Resource, Effect } from "../internal";
import yaml from 'js-yaml';
import { logger } from "../utils";
export class Compiler {
    constructor(resource) {
        this.eventEmitter = new EventEmitter();
        this.resource = resource;
        this.template.hooks.map(hook => hook.appendTo(this.eventEmitter));
    }
    get template() {
        return this.resource.template;
    }
    get project() {
        return this.resource.project;
    }
    emit(name) {
        this.eventEmitter.emit(name);
    }
    prompt(force = false) {
        return __awaiter(this, void 0, void 0, function* () {
            const { template, project } = this;
            let { questions } = template;
            const namesOfQuestions = questions.map(prop('name'));
            if (project.answers && !force) {
                const answers = project.answers;
                questions = questions.filter(question => !question.answered(answers));
            }
            if (!questions.length)
                return;
            logger.info('Please answer the questions of template.');
            let answers = yield Effect.prompter(questions);
            if (project.answers)
                answers = mergeDeepLeft(answers, project.answers);
            project.answers = pick(namesOfQuestions, answers);
        });
    }
    render(options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.prompt(options.ignoreAnswered);
            yield this.template.render(this.resource);
            yield this.generateMilirc();
            this.emit('rendered');
        });
    }
    check(options) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.template.check(this.resource, options);
        });
    }
    generateMilirc() {
        return __awaiter(this, void 0, void 0, function* () {
            const milirc = yaml.safeDump(this.resource.milirc, { skipInvalid: true });
            yield Effect.fs.writeFile(join(this.project.path, '.milirc.yml'), milirc, 'utf8');
        });
    }
}
