/// <reference types="node" />
import { WriteFileOptions, CopyOptions, EnsureOptions, ReadOptions, Stats, WriteOptions } from 'fs-extra';
import { Prompter } from './question';
export interface FileSystem {
    writeFile: (file: string, data: any, options?: WriteFileOptions | string) => Promise<void>;
    readFile: (file: string, encoding: string) => Promise<string>;
    pathExists: (path: string) => Promise<boolean>;
    remove: (dir: string) => Promise<void>;
    emptyDir: (path: string) => Promise<void>;
    copy: (src: string, dest: string, options?: CopyOptions) => Promise<void>;
    ensureDir: (path: string, options?: EnsureOptions | number) => Promise<void>;
    readJSON: (file: string, options?: ReadOptions) => Promise<any>;
    writeJSON: (file: string, object: any, options?: WriteOptions) => Promise<void>;
    stat: (path: string | Buffer) => Promise<Stats>;
    readdir: (path: string | Buffer) => Promise<string[]>;
}
export interface Logger {
    trace(message: any): void;
    debug(message: any): void;
    info(message: any): void;
    warn(message: any): void;
    error(message: any): void;
    fatal(message: any): void;
}
export interface EffectOptions {
    fs?: FileSystem;
    prompter?: Prompter;
    logger?: Logger;
}
export declare class Effect {
    static fs: FileSystem;
    static prompter: Prompter;
    static logger: Logger;
    static replace(options?: EffectOptions): void;
}
