var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { relative, join } from 'path';
import { Handler, Resource, CompiledFile, Effect } from "../../internal";
import { inferEncodingNormally } from "../../infer-encoding";
export class File {
    constructor(path, inferEncoding = inferEncodingNormally, handler) {
        this.templatePath = path;
        this.inferEncoding = inferEncoding;
        this.handler = handler;
    }
    compile(resource) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.readTemplateFile();
            const { templatePath } = this;
            let projectPath = templatePath;
            projectPath = relative(resource.template.path, this.templatePath);
            projectPath = join(resource.project.path, projectPath);
            if (this.handler) {
                projectPath = yield this.handler.genPath(projectPath, resource);
            }
            const content = yield this.readTemplateFile();
            const encoding = this.inferEncoding(this.templatePath);
            const projectFileExisted = yield Effect.fs.pathExists(projectPath);
            const compiledFile = new CompiledFile(templatePath, content, encoding, projectPath, resource, projectFileExisted);
            if (this.handler)
                yield this.handler.genFile(compiledFile, resource);
            return compiledFile;
        });
    }
    readTemplateFile() {
        return __awaiter(this, void 0, void 0, function* () {
            const { templatePath, inferEncoding } = this;
            const encoding = inferEncoding(templatePath);
            return yield Effect.fs.readFile(templatePath, encoding);
        });
    }
}
