var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { buildInHandlers } from "../internal";
const identity = (path) => __awaiter(void 0, void 0, void 0, function* () { return path; });
export class Handler {
    constructor(genFile, genPath = identity) {
        this.genFile = genFile;
        this.genPath = genPath;
    }
    static compose(handlers) {
        const genFile = (file, config) => __awaiter(this, void 0, void 0, function* () {
            for (const handler of handlers) {
                yield handler.genFile(file, config);
            }
        });
        const genPath = (string, config) => __awaiter(this, void 0, void 0, function* () {
            let str = string;
            for (const handler of handlers) {
                str = yield handler.genPath(str, config);
            }
            return str;
        });
        return new Handler(genFile, genPath);
    }
    static find(name) {
        if (name in buildInHandlers)
            return buildInHandlers[name];
        return null;
    }
    static format(handler) {
        if (typeof handler === 'string') {
            const h = Handler.find(handler);
            if (h)
                return h;
            else
                throw new Error(`Cannot find ${handler} handler `);
        }
        else if (typeof handler === 'function') {
            const h = handler(buildInHandlers);
            if (h instanceof Handler)
                return h;
        }
        else if (typeof handler === 'object') {
            return new Handler(handler.genFile, handler.genPath);
        }
        throw new TypeError('Cannot format handler');
    }
}
