var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import git from 'simple-git/promise';
import { basename } from 'path';
import { loadMilirc, loadNpmConfig } from "../loader";
import { Repository, GitRepository, MissingFileError } from "../internal";
import { hasPath } from 'ramda';
import { isRootDirOfRepo } from "../utils";
export class Project {
    constructor(path, name = '', repository, answers) {
        this.path = path;
        this.name = name;
        this.repository = repository;
        this.answers = answers;
    }
    getTemplateRepo() {
        return __awaiter(this, void 0, void 0, function* () {
            const milirc = yield loadMilirc(this.path);
            if (!milirc)
                throw new Error('Cannot load milirc');
            if (!hasPath(['template', 'repository'], milirc)) {
                throw new Error('Cannot find repository config in .milirc');
            }
            const version = milirc.template.version;
            const repoStr = milirc.template.repository;
            const repo = yield Repository.format(repoStr);
            if (version)
                yield repo.checkout(version);
            return repo;
        });
    }
    static load(path) {
        return __awaiter(this, void 0, void 0, function* () {
            const milirc = yield loadMilirc(path);
            let repo;
            let answers;
            if (yield isRootDirOfRepo(path)) {
                const remotes = yield git(path).getRemotes(true);
                if (remotes && remotes.length && remotes[0].refs.push)
                    repo = new GitRepository(remotes[0].refs.push);
            }
            if (milirc && milirc.answers)
                answers = milirc.answers;
            try {
                const npmConfig = yield loadNpmConfig(path);
                if (npmConfig.repository && npmConfig.repository.type === 'git') {
                    repo = new GitRepository(npmConfig.repository.url);
                }
                return new Project(path, npmConfig.name, repo, answers);
            }
            catch (error) {
                if (error instanceof MissingFileError) {
                    return new Project(path, basename(path), repo, answers);
                }
                else {
                    throw error;
                }
            }
        });
    }
}
