var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import childProcess from 'child_process';
import { promisify } from 'util';
import { join } from 'path';
import { Repository } from "./repository";
import { logger } from "../../utils";
import { TEMPLATE_STORAGE } from "../../consts";
import { Effect } from "../../internal";
const genIndexFile = (name) => `
const { join } = require('path')
const config = require('${name}')
const path = join('./node_modules/${name}', config.path)

module.exports = { ...config, path }
`;
const exec = promisify(childProcess.exec);
export class NpmRepository extends Repository {
    constructor(name) {
        super();
        this.type = 'npm';
        this.name = name;
        const matched = name.match(/@(.*)\//);
        if (matched)
            this.scope = matched[1];
        else
            this.scope = null;
    }
    get record() {
        return `npm:${this.name}`;
    }
    get storage() {
        return join(TEMPLATE_STORAGE, encodeURIComponent(this.name), this.version || 'noversion');
    }
    getVersions() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.versions)
                return this.versions;
            const { stdout, stderr } = yield exec(`npm view ${this.name} versions  --json`);
            if (stderr)
                console.error(stderr);
            return JSON.parse(stdout).reverse();
        });
    }
    download() {
        return __awaiter(this, void 0, void 0, function* () {
            const { name, version, storage } = this;
            if (version === 'default' || !version)
                throw new Error('Please checkout version before install npm template');
            const repositoryExisted = yield Effect.fs.pathExists(storage);
            if (repositoryExisted) {
                logger.info('use the cache of template');
                return;
            }
            logger.info(`install ${name} template from npm...`);
            yield Effect.fs.emptyDir(storage);
            yield Effect.fs.writeJSON(join(storage, 'package.json'), {
                main: 'index.js',
                description: '',
                license: 'MIT',
            });
            yield Effect.fs.writeFile(join(storage, 'index.js'), genIndexFile(name));
            yield Effect.fs.writeFile(join(storage, '.npmrc'), 'package-lock=false');
            const command = `npm install ${name}@${version}`;
            yield exec(command, { cwd: storage });
        });
    }
    existed() {
        return __awaiter(this, void 0, void 0, function* () {
            const out = yield exec(`npm view ${this.name} --json`);
            return !out.stderr;
        });
    }
}
