var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { Repository, Resource, Project, Effect, EffectOptions } from "../internal";
import { checkWorkDir } from "../utils";
export default (options) => __awaiter(void 0, void 0, void 0, function* () {
    const cwd = options.cwd || process.cwd();
    const name = options.name;
    const noDeps = options.noDeps || false;
    const version = options.version || 'latest';
    const force = options.force || false;
    if (!options.repository)
        throw new TypeError('options.repository is required for `mili.init(options)`');
    Effect.replace(options.effect);
    if (force)
        yield checkWorkDir(cwd);
    const repo = yield Repository.format(options.repository);
    if (version)
        yield repo.checkout(version);
    const template = yield repo.install({ noDeps });
    const project = yield Project.load(cwd);
    if (options.name)
        project.name = name;
    const resource = new Resource('init', project, template);
    const compiler = yield resource.compile();
    yield compiler.render({ ignoreAnswered: true });
    yield compiler.emit('initialized');
});
