var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { Project, Resource, Effect, EffectOptions } from "../internal";
import { recursiveExecte, logger, checkWorkDir } from "../utils";
import semver from 'semver';
export default recursiveExecte((options) => __awaiter(void 0, void 0, void 0, function* () {
    const cwd = options.cwd || process.cwd();
    const noDeps = options.noDeps || false;
    const force = options.force || false;
    Effect.replace(options.effect);
    if (!force)
        yield checkWorkDir(cwd);
    const project = yield Project.load(cwd);
    const repo = yield project.getTemplateRepo();
    if (yield repo.isLatest()) {
        const message = 'The template is already the latest version';
        if (!force) {
            logger.info(message);
            return;
        }
        logger.warn(message);
    }
    const currentVersion = repo.version;
    yield repo.checkout('latest');
    if (currentVersion && currentVersion !== 'default' && repo.version && semver.diff(currentVersion, repo.version) === 'major') {
        logger.warn(`The template upgrade a major version from ${currentVersion} to ${repo.version}. Please view the template breaking change first.`);
    }
    const template = yield repo.install({ noDeps });
    const resource = new Resource('upgrade', project, template);
    const compiler = yield resource.compile();
    yield compiler.render();
    yield compiler.emit('upgraded');
}));
