var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { extname, basename } from 'path';
import { FileGenerator, Handler } from "../../internal";
import mergeJson from "./merge-json";
import mergeIgnore from "./merge-ignore";
import mergeYaml from "./merge-yaml";
import mergeBabelrc from "./merge-babel";
const jsonFileExts = ['.json'];
const yamlFileExts = ['.yaml', '.yml'];
const ignoreFilenames = ['.gitignore', '.npmignore', '.npmrc', '.eslintignore'];
const babelrcFilenames = ['.babelrc'];
const notSupportErrorMessage = (filename, projectPath) => [
    `Merge files of this type(${filename}) are not supported by merge handler.`,
    'Please change the handler and feedback this question to the developer.',
    `Path: ${projectPath}`,
].join('\n');
const genFile = (file, resource) => __awaiter(void 0, void 0, void 0, function* () {
    const { projectPath } = file;
    const ext = extname(projectPath);
    const filename = basename(projectPath);
    if (jsonFileExts.includes(ext))
        yield mergeJson(file, resource);
    else if (yamlFileExts.includes(ext))
        yield mergeYaml(file, resource);
    else if (ignoreFilenames.includes(filename))
        yield mergeIgnore(file, resource);
    else if (babelrcFilenames.includes(filename))
        yield mergeBabelrc(file, resource);
    else
        throw new Error(notSupportErrorMessage(filename, projectPath));
});
export default new Handler(genFile);
