import { Encoding, InferEncodingFunc } from "../consts";
import inferEncodingNormally from "./infer-encoding-normally";
export default function (mapping) {
    const encodingRegExpMap = Object.entries(mapping)
        .map(([key, value]) => {
        if (typeof value === 'string') {
            const pattern = new RegExp(value);
            return { encoding: key, match: v => pattern.test(v) };
        }
        else if (typeof value === 'function') {
            return { encoding: key, match: v => value(v) };
        }
        throw new Error('Encoding mapping should be function or regexp');
    });
    return path => {
        const item = encodingRegExpMap.find(item => item.match(path));
        if (item && Object.values(Encoding).includes(item.encoding))
            return item.encoding;
        else
            return inferEncodingNormally(path);
    };
}
