var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { join, isAbsolute, dirname, resolve } from 'path';
import glob from 'micromatch';
import { Effect } from "../internal";
import { uniq, union } from 'ramda';
import isChildPathOf from "./is-child-path-of";
const execteFuncInSubproject = (func, dir, options, errors) => __awaiter(void 0, void 0, void 0, function* () {
    const stats = yield Effect.fs.stat(dir);
    if (!stats.isDirectory())
        return;
    if (yield Effect.fs.pathExists(join(dir, '.milirc.yml'))) {
        Effect.logger.info(`Find project ${dir}`);
        try {
            const newOptions = Object.assign(Object.assign({}, options), { cwd: dir });
            yield func(newOptions);
        }
        catch (e) {
            errors.push({ dir, message: e.message });
        }
        finally {
            console.log('\n');
        }
    }
    let folders = yield Effect.fs.readdir(dir);
    folders = folders
        .map(filename => join(dir, filename))
        .filter(filepath => !glob.isMatch(filepath, options.ignore));
    /**
     * NOTE: There should upgrade one by one,
     *       because it's possible that two of these projects were used same template,
     *       resulting in template download conflict.
     */
    for (const folder of folders) {
        yield execteFuncInSubproject(func, folder, options, errors);
    }
});
function dirOfFiles(files, root = '/') {
    if (!files.length)
        return [];
    const dirs = files
        .map(filepath => dirname(filepath))
        .filter(filepath => filepath !== root);
    const uniqDirs = uniq(dirs);
    const parentDirs = dirOfFiles(uniqDirs, root);
    return union(parentDirs, dirs);
}
/**
 * Check the dir of files that include .milirc.
 * It is useful for lint-stage.
 */
const execteFuncInParentProject = (func, files, options, errors) => __awaiter(void 0, void 0, void 0, function* () {
    const baseUrl = options.cwd ? resolve(options.cwd) : process.cwd();
    const dirs = dirOfFiles(files)
        .filter(isChildPathOf(baseUrl, true));
    for (const dir of dirs) {
        if (yield Effect.fs.pathExists(join(dir, '.milirc.yml'))) {
            Effect.logger.info(`Find project ${dir}`);
            try {
                const newOptions = Object.assign(Object.assign({}, options), { cwd: dir });
                yield func(newOptions);
            }
            catch (e) {
                errors.push({ dir, message: e.message });
            }
            finally {
                console.log('\n');
            }
        }
    }
});
const handleErrors = (errors) => {
    if (errors.length) {
        errors.forEach(error => {
            Effect.logger.error([
                '',
                `Fail: ${error.dir}.`,
                `Because: ${error.message}`,
                '',
            ].join('\n'));
        });
        throw new Error('Please fix the error.');
    }
};
export default (func) => (options) => __awaiter(void 0, void 0, void 0, function* () {
    const { cwd = process.cwd(), ignore = [], recursive = false, files = [], } = options;
    if (files.length) {
        const absoluteFiles = files.map((item) => {
            if (!isAbsolute(item))
                return join(cwd, item);
            return item;
        });
        const errors = [];
        yield execteFuncInParentProject(func, absoluteFiles, options, errors);
        handleErrors(errors);
    }
    else if (!recursive) {
        yield func(options);
    }
    else {
        const absolutePathIgnored = ignore.map((item) => {
            if (!isAbsolute(item))
                return join(cwd, item);
            return item;
        });
        const newOptions = Object.assign(Object.assign({}, options), { ignore: absolutePathIgnored });
        const errors = [];
        yield execteFuncInSubproject(func, cwd, newOptions, errors);
        handleErrors(errors);
    }
});
