"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const events_1 = require("events");
const ramda_1 = require("ramda");
const path_1 = require("path");
const internal_1 = require("../internal");
const js_yaml_1 = __importDefault(require("js-yaml"));
const utils_1 = require("../utils");
class Compiler {
    constructor(resource) {
        this.eventEmitter = new events_1.EventEmitter();
        this.resource = resource;
        this.template.hooks.map(hook => hook.appendTo(this.eventEmitter));
    }
    get template() {
        return this.resource.template;
    }
    get project() {
        return this.resource.project;
    }
    emit(name) {
        this.eventEmitter.emit(name);
    }
    prompt(force = false) {
        return __awaiter(this, void 0, void 0, function* () {
            const { template, project } = this;
            let { questions } = template;
            const namesOfQuestions = questions.map(ramda_1.prop('name'));
            if (project.answers && !force) {
                const answers = project.answers;
                questions = questions.filter(question => !question.answered(answers));
            }
            if (!questions.length)
                return;
            utils_1.logger.info('Please answer the questions of template.');
            let answers = yield internal_1.Effect.prompter(questions);
            if (project.answers)
                answers = ramda_1.mergeDeepLeft(answers, project.answers);
            project.answers = ramda_1.pick(namesOfQuestions, answers);
        });
    }
    render(options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.prompt(options.ignoreAnswered);
            yield this.template.render(this.resource);
            yield this.generateMilirc();
            this.emit('rendered');
        });
    }
    check(options) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.template.check(this.resource, options);
        });
    }
    generateMilirc() {
        return __awaiter(this, void 0, void 0, function* () {
            const milirc = js_yaml_1.default.safeDump(this.resource.milirc, { skipInvalid: true });
            yield internal_1.Effect.fs.writeFile(path_1.join(this.project.path, '.milirc.yml'), milirc, 'utf8');
        });
    }
}
exports.Compiler = Compiler;
