"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const internal_1 = require("../../internal");
const utils_1 = require("../../utils");
const ramda_1 = require("ramda");
class CompiledFile {
    constructor(templatePath, content, encoding, projectPath, resource, projectFileExisted) {
        /** Delete project file */
        this.deleted = false;
        /** Need to render file */
        this.renderable = true;
        /** Additional file information that added by handler */
        this.addition = {};
        this.templatePath = templatePath;
        this.projectPath = projectPath;
        this.projectFileExisted = projectFileExisted;
        this.encoding = encoding;
        this.content = content;
        this.resource = resource;
    }
    getProjectContent() {
        return __awaiter(this, void 0, void 0, function* () {
            const { projectPath, encoding } = this;
            if (!ramda_1.isNil(this.projectContent))
                return this.projectContent;
            if (!this.projectFileExisted)
                throw new Error(`Cannot get content from an unexisted file ${projectPath}.`);
            this.projectContent = yield internal_1.Effect.fs.readFile(projectPath, encoding);
            return this.projectContent;
        });
    }
    render() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.deleted) {
                yield internal_1.Effect.fs.remove(this.projectPath);
                return;
            }
            if (!this.renderable)
                return;
            const { projectPath, content, encoding } = this;
            yield internal_1.Effect.fs.writeFile(projectPath, content, encoding);
        });
    }
    check(options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            const { projectPath, deleted, renderable, projectFileExisted } = this;
            if (deleted && projectFileExisted)
                throw new Error(`${projectPath}: Should be removed`);
            if (!renderable)
                return true;
            if (!projectFileExisted)
                throw new Error(`${projectPath}: Not exist`);
            const projectContent = yield this.getProjectContent();
            if (this.content === projectContent)
                return true;
            if (options.showDiff) {
                const diff = utils_1.diffFile(this.projectPath, projectContent, this.content, { fold: options.fold });
                throw new Error(diff);
            }
            else {
                throw new Error(`${projectPath}: Not match template.`);
            }
        });
    }
}
exports.CompiledFile = CompiledFile;
