"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const path_1 = require("path");
const internal_1 = require("../../internal");
const infer_encoding_1 = require("../../infer-encoding");
class File {
    constructor(path, inferEncoding = infer_encoding_1.inferEncodingNormally, handler) {
        this.templatePath = path;
        this.inferEncoding = inferEncoding;
        this.handler = handler;
    }
    compile(resource) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.readTemplateFile();
            const { templatePath } = this;
            let projectPath = templatePath;
            projectPath = path_1.relative(resource.template.path, this.templatePath);
            projectPath = path_1.join(resource.project.path, projectPath);
            if (this.handler) {
                projectPath = yield this.handler.genPath(projectPath, resource);
            }
            const content = yield this.readTemplateFile();
            const encoding = this.inferEncoding(this.templatePath);
            const projectFileExisted = yield internal_1.Effect.fs.pathExists(projectPath);
            const compiledFile = new internal_1.CompiledFile(templatePath, content, encoding, projectPath, resource, projectFileExisted);
            if (this.handler)
                yield this.handler.genFile(compiledFile, resource);
            return compiledFile;
        });
    }
    readTemplateFile() {
        return __awaiter(this, void 0, void 0, function* () {
            const { templatePath, inferEncoding } = this;
            const encoding = inferEncoding(templatePath);
            return yield internal_1.Effect.fs.readFile(templatePath, encoding);
        });
    }
}
exports.File = File;
