"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const child_process_1 = __importDefault(require("child_process"));
const util_1 = require("util");
const utils_1 = require("../utils");
const exec = util_1.promisify(child_process_1.default.exec);
class Hook {
    /*
     * constructor(eventName: string, listener: string)
     * constructor(eventName: string, listener: Listener)
     */
    constructor(eventName, listener) {
        this.eventName = eventName;
        if (typeof listener === 'string') {
            this.listener = () => __awaiter(this, void 0, void 0, function* () {
                utils_1.logger.info(`run ${eventName} hook...`);
                try {
                    const { stdout, stderr } = yield exec(listener);
                    process.stdout.write(stdout);
                    process.stderr.write(stderr);
                }
                catch (error) {
                    utils_1.logger.error('hook exec error', error);
                }
            });
        }
        else if (typeof listener === 'function') {
            this.listener = (...arg) => __awaiter(this, void 0, void 0, function* () {
                utils_1.logger.info(`run ${eventName} hook...`);
                yield listener(...arg);
            });
        }
        else {
            throw new Error('Hook should be a string or function');
        }
    }
    appendTo(eventEmitter) {
        eventEmitter.addListener(this.eventName, this.listener);
    }
}
exports.Hook = Hook;
