"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const promise_1 = __importDefault(require("simple-git/promise"));
const path_1 = require("path");
const loader_1 = require("../loader");
const internal_1 = require("../internal");
const ramda_1 = require("ramda");
const utils_1 = require("../utils");
class Project {
    constructor(path, name = '', repository, answers) {
        this.path = path;
        this.name = name;
        this.repository = repository;
        this.answers = answers;
    }
    getTemplateRepo() {
        return __awaiter(this, void 0, void 0, function* () {
            const milirc = yield loader_1.loadMilirc(this.path);
            if (!milirc)
                throw new Error('Cannot load milirc');
            if (!ramda_1.hasPath(['template', 'repository'], milirc)) {
                throw new Error('Cannot find repository config in .milirc');
            }
            const version = milirc.template.version;
            const repoStr = milirc.template.repository;
            const repo = yield internal_1.Repository.format(repoStr);
            if (version)
                yield repo.checkout(version);
            return repo;
        });
    }
    static load(path) {
        return __awaiter(this, void 0, void 0, function* () {
            const milirc = yield loader_1.loadMilirc(path);
            let repo;
            let answers;
            if (yield utils_1.isRootDirOfRepo(path)) {
                const remotes = yield promise_1.default(path).getRemotes(true);
                if (remotes && remotes.length && remotes[0].refs.push)
                    repo = new internal_1.GitRepository(remotes[0].refs.push);
            }
            if (milirc && milirc.answers)
                answers = milirc.answers;
            try {
                const npmConfig = yield loader_1.loadNpmConfig(path);
                if (npmConfig.repository && npmConfig.repository.type === 'git') {
                    repo = new internal_1.GitRepository(npmConfig.repository.url);
                }
                return new Project(path, npmConfig.name, repo, answers);
            }
            catch (error) {
                if (error instanceof internal_1.MissingFileError) {
                    return new Project(path, path_1.basename(path), repo, answers);
                }
                else {
                    throw error;
                }
            }
        });
    }
}
exports.Project = Project;
