"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const ramda_1 = require("ramda");
class Question {
    constructor(options) {
        this.type = 'input';
        this.name = '';
        this.message = '';
        this.choices = [];
        this.validate = ramda_1.T;
        this.filter = ramda_1.identity;
        this.transformer = ramda_1.identity;
        this.when = ramda_1.T;
        this.pageSize = 0;
        this.prefix = '';
        this.suffix = '';
        if (options.type)
            this.type = options.type;
        if (options.name)
            this.name = options.name;
        else
            throw new TypeError('name should not be empty');
        if (options.message)
            this.message = options.message;
        if (options.default)
            this.default = options.default;
        if (options.choices)
            this.choices = options.choices;
        if (options.validate)
            this.validate = options.validate;
        if (options.filter)
            this.filter = options.filter;
        if (options.transformer)
            this.transformer = options.transformer;
        if (options.when)
            this.when = options.when;
        if (options.pageSize)
            this.pageSize = options.pageSize;
        if (options.prefix)
            this.prefix = options.prefix;
        if (options.suffix)
            this.suffix = options.suffix;
    }
    includesByChoices(value, answers) {
        let choices = this.choices;
        if (typeof choices === 'function')
            choices = choices(answers);
        return choices
            .filter(choice => typeof choice !== 'object' || !choice.disabled)
            .some(choice => {
            if (typeof choice === 'object')
                return choice.value === value;
            return choice === value;
        });
    }
    answered(answers) {
        const { name, type } = this;
        if (!Object.keys(answers).includes(name))
            return false;
        const answer = answers[name];
        if (type === 'input' || type === 'password')
            return typeof answer === 'string';
        if (type === 'number')
            return typeof answer === 'number';
        if (type === 'confirm')
            return typeof answer === 'boolean';
        if (type === 'list' || type === 'rawlist' || type === 'expand')
            return this.includesByChoices(answer, answers);
        if (type === 'checkbox' && Array.isArray(answer))
            return answer.every(value => this.includesByChoices(value, answers));
        return false;
    }
}
exports.Question = Question;
