"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const promise_1 = __importDefault(require("simple-git/promise"));
const semver_1 = __importDefault(require("semver"));
const path_1 = require("path");
const repository_1 = require("./repository");
const consts_1 = require("../../consts");
const utils_1 = require("../../utils");
const internal_1 = require("../../internal");
const simple_git_1 = require("simple-git");
class LocalRepository extends repository_1.Repository {
    constructor(str) {
        super();
        this.absolute = false;
        this.type = 'local';
        this.name = path_1.basename(str);
        const path = str;
        const cwd = process.cwd();
        if (path_1.isAbsolute(path)) {
            this.absolute = true;
            this.path = path;
        }
        else {
            this.path = path_1.join(cwd, path);
        }
    }
    get record() {
        if (this.absolute)
            return this.path;
        /** the path saved in .milirc should be relative to the output folder, rather than process.cwd() */
        return projectPath => {
            const relativePath = path_1.relative(projectPath, this.path);
            if (utils_1.isRelativePath(relativePath))
                return relativePath;
            else
                return `./${relativePath}`;
        };
    }
    get storage() {
        return path_1.join(consts_1.TEMPLATE_STORAGE, encodeURIComponent(this.path), this.version || 'noversion');
    }
    getVersions() {
        return __awaiter(this, void 0, void 0, function* () {
            const { path } = this;
            if (this.versions)
                return this.versions;
            if (!(yield promise_1.default(path).checkIsRepo()) || !(yield utils_1.isRootDirOfRepo(path))) {
                this.versions = [];
                return this.versions;
            }
            const tags = yield promise_1.default(path).tags();
            if (!tags.all.length) {
                utils_1.logger.warn([
                    'Cannot get template versions, May be caused by the following reasons:',
                    `1. repository is not a mili template(${path})`,
                    '2. template have not a valid tag to mark the version(e.g. v1.0.0)',
                    `3. cannot get versions by command: \`git tags ${path}\``,
                ].join('\n'));
            }
            this.versions = tags.all
                .map(tag => semver_1.default.clean(tag) || '')
                .filter(tag => Boolean(tag))
                .reverse();
            return this.versions;
        });
    }
    download() {
        return __awaiter(this, void 0, void 0, function* () {
            const { path, version, storage } = this;
            yield internal_1.Effect.fs.emptyDir(storage);
            utils_1.logger.info(`copy template from ${path}`);
            yield internal_1.Effect.fs.copy(path, storage);
            if (utils_1.isRootDirOfRepo(storage) && version && version !== 'default') {
                yield promise_1.default(storage).reset(simple_git_1.ResetMode.HARD);
                yield promise_1.default(storage).checkout(`v${version}`);
                utils_1.logger.info(`template version: ${version}`);
            }
            else if (version !== 'default') {
                utils_1.logger.warn('Version is unset, use the default files');
            }
        });
    }
    existed() {
        return __awaiter(this, void 0, void 0, function* () {
            return yield utils_1.dirExist(this.path);
        });
    }
}
exports.LocalRepository = LocalRepository;
