"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const child_process_1 = __importDefault(require("child_process"));
const util_1 = require("util");
const path_1 = require("path");
const repository_1 = require("./repository");
const utils_1 = require("../../utils");
const consts_1 = require("../../consts");
const internal_1 = require("../../internal");
const genIndexFile = (name) => `
const { join } = require('path')
const config = require('${name}')
const path = join('./node_modules/${name}', config.path)

module.exports = { ...config, path }
`;
const exec = util_1.promisify(child_process_1.default.exec);
class NpmRepository extends repository_1.Repository {
    constructor(name) {
        super();
        this.type = 'npm';
        this.name = name;
        const matched = name.match(/@(.*)\//);
        if (matched)
            this.scope = matched[1];
        else
            this.scope = null;
    }
    get record() {
        return `npm:${this.name}`;
    }
    get storage() {
        return path_1.join(consts_1.TEMPLATE_STORAGE, encodeURIComponent(this.name), this.version || 'noversion');
    }
    getVersions() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.versions)
                return this.versions;
            const { stdout, stderr } = yield exec(`npm view ${this.name} versions  --json`);
            if (stderr)
                console.error(stderr);
            return JSON.parse(stdout).reverse();
        });
    }
    download() {
        return __awaiter(this, void 0, void 0, function* () {
            const { name, version, storage } = this;
            if (version === 'default' || !version)
                throw new Error('Please checkout version before install npm template');
            const repositoryExisted = yield internal_1.Effect.fs.pathExists(storage);
            if (repositoryExisted) {
                utils_1.logger.info('use the cache of template');
                return;
            }
            utils_1.logger.info(`install ${name} template from npm...`);
            yield internal_1.Effect.fs.emptyDir(storage);
            yield internal_1.Effect.fs.writeJSON(path_1.join(storage, 'package.json'), {
                main: 'index.js',
                description: '',
                license: 'MIT',
            });
            yield internal_1.Effect.fs.writeFile(path_1.join(storage, 'index.js'), genIndexFile(name));
            yield internal_1.Effect.fs.writeFile(path_1.join(storage, '.npmrc'), 'package-lock=false');
            const command = `npm install ${name}@${version}`;
            yield exec(command, { cwd: storage });
        });
    }
    existed() {
        return __awaiter(this, void 0, void 0, function* () {
            const out = yield exec(`npm view ${this.name} --json`);
            return !out.stderr;
        });
    }
}
exports.NpmRepository = NpmRepository;
