import { Template } from '@/internal';
export declare type RepositoryTypes = 'npm' | 'local' | 'git';
export interface InstallOptions {
    noDeps?: boolean;
}
export declare abstract class Repository {
    type: RepositoryTypes;
    owner: string;
    name: string;
    version?: string;
    protected versions: string[] | null;
    static format(str: string): Promise<Repository>;
    hasVersion(version: string): Promise<boolean>;
    checkout(version?: string): Promise<void>;
    isVerioning(): Promise<boolean>;
    isLatest(): Promise<boolean>;
    private installDeps;
    install(options?: InstallOptions): Promise<Template>;
    abstract get storage(): string;
    abstract get record(): string | ((projectPath: string) => string);
    abstract existed(): Promise<boolean>;
    /**
     * get repository versions list.
     * In the order from new to old.
     */
    abstract getVersions(): Promise<string[]>;
    abstract download(): Promise<void>;
}
