"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const semver_1 = __importDefault(require("semver"));
const internal_1 = require("../internal");
const package_json_1 = require("../../package.json");
const utils_1 = require("../utils");
class Resource {
    constructor(operation, project, template) {
        this.mili = { version: package_json_1.version };
        if (!template.engines) {
            utils_1.logger.warn('The template.engines is not set, you need to check the mili version manually.');
        }
        else if (!semver_1.default.satisfies(this.mili.version, template.engines)) {
            throw new Error([
                `The mili version template need is ${template.engines}`,
                `But mili version used is ${this.mili.version}`,
            ].join('\n'));
        }
        this.operation = operation;
        this.project = project;
        this.template = template;
    }
    get answers() {
        return this.project.answers;
    }
    get milirc() {
        const { mili, project, template, answers } = this;
        let { record } = template.repository;
        if (typeof record === 'function')
            record = record(project.path);
        return {
            mili,
            template: {
                repository: record,
                version: template.repository.version,
            },
            answers,
        };
    }
    compile() {
        return __awaiter(this, void 0, void 0, function* () {
            return new internal_1.Compiler(this);
        });
    }
}
exports.Resource = Resource;
