"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const internal_1 = require("../internal");
const utils_1 = require("../utils");
const semver_1 = __importDefault(require("semver"));
exports.default = utils_1.recursiveExecte((options) => __awaiter(void 0, void 0, void 0, function* () {
    const cwd = options.cwd || process.cwd();
    const noDeps = options.noDeps || false;
    const version = options.version;
    const force = options.force || false;
    internal_1.Effect.replace(options.effect);
    if (!force)
        yield utils_1.checkWorkDir(cwd);
    const project = yield internal_1.Project.load(cwd);
    const repo = yield project.getTemplateRepo();
    if (version && repo.version && version !== 'default' && version !== 'latest' && semver_1.default.lt(version, repo.version)) {
        const message = [
            'The version number setted is lower than the current template version.',
            "If you're sure you want to run this command, rerun it with --force.",
        ].join('\n');
        if (force)
            utils_1.logger.warn(message);
        else
            throw new Error(message);
    }
    if (version)
        yield repo.checkout(version);
    const template = yield repo.install({ noDeps });
    const resource = new internal_1.Resource('update', project, template);
    const compiler = yield resource.compile();
    yield compiler.render();
    yield compiler.emit('updated');
}));
