"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const internal_1 = require("../internal");
const utils_1 = require("../utils");
const semver_1 = __importDefault(require("semver"));
exports.default = utils_1.recursiveExecte((options) => __awaiter(void 0, void 0, void 0, function* () {
    const cwd = options.cwd || process.cwd();
    const noDeps = options.noDeps || false;
    const force = options.force || false;
    internal_1.Effect.replace(options.effect);
    if (!force)
        yield utils_1.checkWorkDir(cwd);
    const project = yield internal_1.Project.load(cwd);
    const repo = yield project.getTemplateRepo();
    if (yield repo.isLatest()) {
        const message = 'The template is already the latest version';
        if (!force) {
            utils_1.logger.info(message);
            return;
        }
        utils_1.logger.warn(message);
    }
    const currentVersion = repo.version;
    yield repo.checkout('latest');
    if (currentVersion && currentVersion !== 'default' && repo.version && semver_1.default.diff(currentVersion, repo.version) === 'major') {
        utils_1.logger.warn(`The template upgrade a major version from ${currentVersion} to ${repo.version}. Please view the template breaking change first.`);
    }
    const template = yield repo.install({ noDeps });
    const resource = new internal_1.Resource('upgrade', project, template);
    const compiler = yield resource.compile();
    yield compiler.render();
    yield compiler.emit('upgraded');
}));
