"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const path_1 = require("path");
const internal_1 = require("../../internal");
const merge_json_1 = __importDefault(require("./merge-json"));
const merge_ignore_1 = __importDefault(require("./merge-ignore"));
const merge_yaml_1 = __importDefault(require("./merge-yaml"));
const merge_babel_1 = __importDefault(require("./merge-babel"));
const jsonFileExts = ['.json'];
const yamlFileExts = ['.yaml', '.yml'];
const ignoreFilenames = ['.gitignore', '.npmignore', '.npmrc', '.eslintignore'];
const babelrcFilenames = ['.babelrc'];
const notSupportErrorMessage = (filename, projectPath) => [
    `Merge files of this type(${filename}) are not supported by merge handler.`,
    'Please change the handler and feedback this question to the developer.',
    `Path: ${projectPath}`,
].join('\n');
const genFile = (file, resource) => __awaiter(void 0, void 0, void 0, function* () {
    const { projectPath } = file;
    const ext = path_1.extname(projectPath);
    const filename = path_1.basename(projectPath);
    if (jsonFileExts.includes(ext))
        yield merge_json_1.default(file, resource);
    else if (yamlFileExts.includes(ext))
        yield merge_yaml_1.default(file, resource);
    else if (ignoreFilenames.includes(filename))
        yield merge_ignore_1.default(file, resource);
    else if (babelrcFilenames.includes(filename))
        yield merge_babel_1.default(file, resource);
    else
        throw new Error(notSupportErrorMessage(filename, projectPath));
});
exports.default = new internal_1.Handler(genFile);
