"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const ramda_1 = require("ramda");
const merge = (a, b) => {
    if (Array.isArray(a) && Array.isArray(b))
        return ramda_1.union(a, b);
    return b;
};
const mergeDeepWithUnionArray = ramda_1.mergeDeepWith(merge);
class Plugin {
    constructor(item) {
        this.name = Array.isArray(item) ? item[0] : item;
        if (Array.isArray(item) && typeof item[1] === 'object')
            this.option = item[1];
        else
            this.option = {};
    }
    get content() {
        if (Object.keys(this.option).length)
            return [this.name, this.option];
        return this.name;
    }
    equal(plugin) {
        return this.name === plugin.name || this.name.includes(plugin.name);
    }
    merge(plugin) {
        const option = mergeDeepWithUnionArray(this.option, plugin.option);
        return new Plugin([this.name, option]);
    }
}
function mergeArray(source = [], overrides = []) {
    const reduction = [];
    const plugins = [...source, ...overrides]
        .map(item => new Plugin(item));
    plugins.forEach(plugin => {
        const index = reduction.findIndex(item => plugin.equal(item));
        if (index !== -1)
            reduction[index] = plugin.merge(reduction[index]);
        else
            reduction.push(plugin);
    });
    return reduction;
}
function babelMerge(source = {}, overrides = {}) {
    const plugins = mergeArray(source.plugins, overrides.plugins);
    const presets = mergeArray(source.presets, overrides.presets);
    const sourceEnv = source.env || {};
    const overridesEnv = overrides.env || {};
    const envs = ramda_1.union(Object.keys(sourceEnv), Object.keys(overridesEnv))
        .map(name => ({
        [name]: babelMerge(sourceEnv[name], overridesEnv[name]),
    }));
    const result = {};
    if (presets.length)
        result.presets = presets.map(plugin => plugin.content);
    if (plugins.length)
        result.plugins = plugins.map(plugin => plugin.content);
    if (envs.length)
        result.env = ramda_1.mergeAll(envs);
    return result;
}
const mergeBabelrc = (file) => __awaiter(void 0, void 0, void 0, function* () {
    let { content } = file;
    const beginMatched = content.match(/^\s*/g);
    const beginBlank = beginMatched ? beginMatched[0] : '';
    const endMatched = content.match(/\s*$/g);
    const endBlank = endMatched ? endMatched[0] : '';
    let result;
    try {
        content = JSON.parse(file.content);
    }
    catch (e) {
        throw new Error([
            'The template file and the current file failed to merge due to a json syntax error in the template file.',
            'The current file will be overwritten directly by the template file.',
            `path: ${file.templatePath}`,
        ].join('\n'));
    }
    if (file.projectFileExisted) {
        let projectContent = yield file.getProjectContent();
        try {
            projectContent = JSON.parse(projectContent);
        }
        catch (e) {
            throw new Error([
                'The template file and the current file failed to merge due to a json syntax error in the current file.',
                'The current file will be overwritten directly by the template file.',
                `path: ${file.projectPath}`,
            ].join('\n'));
        }
        result = JSON.stringify(babelMerge(projectContent, content), null, '  ');
    }
    else {
        result = JSON.stringify(babelMerge(content, {}), null, '  ');
    }
    file.content = `${beginBlank}${result}${endBlank}`;
});
exports.default = mergeBabelrc;
