"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const ramda_1 = require("ramda");
const defaultBucketName = Symbol();
const classifyIgnore = (list) => {
    let last = { name: defaultBucketName, values: [] };
    const bucket = [last];
    list.forEach(item => {
        if (/^#/.test(item)) {
            const name = item.substring(1);
            /** comment adjacent comments */
            if (!last.values.length && typeof last.name !== 'symbol') {
                last.name = `${last.name}\n${name}`;
                return;
            }
            const pair = bucket.find(item => item.name === name);
            if (pair) {
                last = pair;
            }
            else {
                last = { name, values: [] };
                bucket.push(last);
            }
        }
        else if (item) {
            if (!last.values.includes(item))
                last.values.push(item);
        }
    });
    return bucket;
};
const mergeBucket = (b1, b2) => {
    const bucket = b1.map(item => (Object.assign({}, item)));
    b2.forEach(pair => {
        const sameOne = bucket.find(item => item.name === pair.name);
        if (!sameOne) {
            bucket.push(pair);
        }
        else {
            pair.values.forEach(value => {
                if (!(sameOne.values.includes(value)))
                    sameOne.values = sameOne.values.concat(value);
            });
        }
    });
    return bucket;
};
const uniqBucket = (b1, b2) => {
    const allValues = ramda_1.unnest(b2.map(item => item.values));
    return b1.map(pair => {
        const values = pair.values.filter(item => !allValues.includes(item));
        return { name: pair.name, values };
    });
};
const renderBucket = (bucket) => bucket
    .filter(pair => (pair.name !== defaultBucketName || pair.values.length))
    .map(({ name, values }) => {
    let str = '';
    if (typeof name === 'string')
        str += name.replace(/^(.*)(\n|$)/mg, '#$1\n');
    str += values.join('\n');
    return str;
})
    .join('\n\n');
const mergeIgnore = (file) => __awaiter(void 0, void 0, void 0, function* () {
    const templateIgnoreList = file.content.split('\n');
    let projectContent = '';
    if (file.projectFileExisted)
        projectContent = yield file.getProjectContent();
    const projectIgnoreList = projectContent.split('\n');
    const templateIgnoreBucket = classifyIgnore(templateIgnoreList);
    const projectIgnoreBucket = classifyIgnore(projectIgnoreList);
    const uniquedBucket = uniqBucket(projectIgnoreBucket, templateIgnoreBucket);
    const bucket = mergeBucket(templateIgnoreBucket, uniquedBucket);
    const result = renderBucket(bucket);
    const beginMatched = file.content.match(/^\s*/g);
    const beginBlank = beginMatched ? beginMatched[0] : '';
    const endMatched = file.content.match(/\s*$/g);
    const endBlank = endMatched ? endMatched[0] : '';
    file.content = `${beginBlank}${result}${endBlank}`;
});
exports.default = mergeIgnore;
