"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const cosmiconfig_1 = require("cosmiconfig");
const semver_1 = __importDefault(require("semver"));
const path_1 = require("path");
const ajv_1 = __importDefault(require("ajv"));
const utils_1 = require("../utils");
const schema_1 = require("../schema");
const internal_1 = require("../internal");
const ramda_1 = require("ramda");
const ajv = new ajv_1.default({ useDefaults: true, nullable: true });
const validate = ajv.compile(schema_1.MilircSchema);
const explorer = cosmiconfig_1.cosmiconfig('mili');
exports.default = (cwd) => __awaiter(void 0, void 0, void 0, function* () {
    const filepath = path_1.join(cwd, '.milirc.yml');
    if (!(yield internal_1.Effect.fs.pathExists(filepath)))
        return null;
    /**
     * NOTE: don't change the object that return by cosmiconfig.
     *       It was cache by cosmiconfig
     */
    const result = yield explorer.load(filepath);
    if (!result || !result.config)
        return null;
    const config = ramda_1.clone(result.config);
    if (!config)
        throw new Error('Cannot load .milirc.yml');
    /* Should before validate. mili@1 config won't pass validator */
    if (config.mili && semver_1.default.lt(config.mili.version, '2.0.0'))
        throw new Error('Never support auto upgrade from mili@1');
    const valid = validate(config);
    if (!valid)
        throw new Error(ajv.errorsText(validate.errors, { dataVar: 'milirc' }));
    const milirc = config;
    /** The relative template path saved in .milirc is relative to the dir of .milirc */
    if (utils_1.isRelativePath(config.template.repository)) {
        config.template.repository = utils_1.relativePath(process.cwd(), path_1.join(cwd, config.template.repository));
    }
    return milirc;
});
