"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const path_1 = require("path");
const internal_1 = require("../internal");
const utils_1 = require("../utils");
exports.default = (path) => __awaiter(void 0, void 0, void 0, function* () {
    const packageJsonPath = path_1.join(path, 'package.json');
    const exist = yield internal_1.Effect.fs.pathExists(packageJsonPath);
    if (!exist)
        throw new internal_1.MissingFileError(path);
    try {
        const config = yield internal_1.Effect.fs.readJSON(packageJsonPath);
        if (typeof config.repository === 'string') {
            if (utils_1.validateGitRepoUrl(config.repository)) {
                config.repository = { type: 'git', url: config.repository };
            }
            else {
                config.repository = { url: config.repository };
            }
        }
        if (typeof config.repository === 'object') {
            if (config.repository.type === 'git' && config.repository.url) {
                const url = config.repository.url.replace(/^git\+/, '');
                config.repository = { type: 'git', url };
            }
            else {
                config.repository = { url: config.repository.url };
            }
        }
        else {
            config.repository = undefined;
        }
        if (typeof config.main !== 'string')
            config.main = undefined;
        return config;
    }
    catch (e) {
        throw new Error([
            `Cannot load package.json from ${path}.`,
            'Maybe syntax error in package.json',
        ].join('\n'));
    }
});
