"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const path_1 = require("path");
const promise_1 = __importDefault(require("simple-git/promise"));
const utils_1 = require("./");
const reminder = [
    'This command may cause some files to be overwritten',
    "If you're sure you want to run this command, rerun it with --force.",
].join('\n');
const isWorkDirClean = (path) => __awaiter(void 0, void 0, void 0, function* () {
    const isRepo = yield promise_1.default(path).checkIsRepo();
    if (!isRepo)
        throw new Error('The work directory checked should be an repository');
    const { files } = yield promise_1.default(path).status();
    let toplevel = yield promise_1.default(path).revparse(['--show-toplevel']);
    toplevel = toplevel.replace(/\n$/, '');
    return !files
        .map(file => path_1.join(toplevel, file.path))
        .filter(utils_1.isChildPathOf(path))
        .length;
});
exports.default = (path) => __awaiter(void 0, void 0, void 0, function* () {
    const isRepo = yield promise_1.default(path).checkIsRepo();
    if (!isRepo && !(yield utils_1.isEmptyDir(path))) {
        throw new Error([
            'The project directory is not a git repository and is a non-empty folder.',
            reminder,
        ].join('\n'));
    }
    else if (isRepo && !(yield isWorkDirClean(path))) {
        throw new Error([
            'Git working directory not clean',
            reminder,
        ].join('\n'));
    }
});
