"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const path_1 = require("path");
const micromatch_1 = __importDefault(require("micromatch"));
const internal_1 = require("../internal");
const ramda_1 = require("ramda");
const is_child_path_of_1 = __importDefault(require("./is-child-path-of"));
const execteFuncInSubproject = (func, dir, options, errors) => __awaiter(void 0, void 0, void 0, function* () {
    const stats = yield internal_1.Effect.fs.stat(dir);
    if (!stats.isDirectory())
        return;
    if (yield internal_1.Effect.fs.pathExists(path_1.join(dir, '.milirc.yml'))) {
        internal_1.Effect.logger.info(`Find project ${dir}`);
        try {
            const newOptions = Object.assign(Object.assign({}, options), { cwd: dir });
            yield func(newOptions);
        }
        catch (e) {
            errors.push({ dir, message: e.message });
        }
        finally {
            console.log('\n');
        }
    }
    let folders = yield internal_1.Effect.fs.readdir(dir);
    folders = folders
        .map(filename => path_1.join(dir, filename))
        .filter(filepath => !micromatch_1.default.isMatch(filepath, options.ignore));
    /**
     * NOTE: There should upgrade one by one,
     *       because it's possible that two of these projects were used same template,
     *       resulting in template download conflict.
     */
    for (const folder of folders) {
        yield execteFuncInSubproject(func, folder, options, errors);
    }
});
function dirOfFiles(files, root = '/') {
    if (!files.length)
        return [];
    const dirs = files
        .map(filepath => path_1.dirname(filepath))
        .filter(filepath => filepath !== root);
    const uniqDirs = ramda_1.uniq(dirs);
    const parentDirs = dirOfFiles(uniqDirs, root);
    return ramda_1.union(parentDirs, dirs);
}
/**
 * Check the dir of files that include .milirc.
 * It is useful for lint-stage.
 */
const execteFuncInParentProject = (func, files, options, errors) => __awaiter(void 0, void 0, void 0, function* () {
    const baseUrl = options.cwd ? path_1.resolve(options.cwd) : process.cwd();
    const dirs = dirOfFiles(files)
        .filter(is_child_path_of_1.default(baseUrl, true));
    for (const dir of dirs) {
        if (yield internal_1.Effect.fs.pathExists(path_1.join(dir, '.milirc.yml'))) {
            internal_1.Effect.logger.info(`Find project ${dir}`);
            try {
                const newOptions = Object.assign(Object.assign({}, options), { cwd: dir });
                yield func(newOptions);
            }
            catch (e) {
                errors.push({ dir, message: e.message });
            }
            finally {
                console.log('\n');
            }
        }
    }
});
const handleErrors = (errors) => {
    if (errors.length) {
        errors.forEach(error => {
            internal_1.Effect.logger.error([
                '',
                `Fail: ${error.dir}.`,
                `Because: ${error.message}`,
                '',
            ].join('\n'));
        });
        throw new Error('Please fix the error.');
    }
};
exports.default = (func) => (options) => __awaiter(void 0, void 0, void 0, function* () {
    const { cwd = process.cwd(), ignore = [], recursive = false, files = [], } = options;
    if (files.length) {
        const absoluteFiles = files.map((item) => {
            if (!path_1.isAbsolute(item))
                return path_1.join(cwd, item);
            return item;
        });
        const errors = [];
        yield execteFuncInParentProject(func, absoluteFiles, options, errors);
        handleErrors(errors);
    }
    else if (!recursive) {
        yield func(options);
    }
    else {
        const absolutePathIgnored = ignore.map((item) => {
            if (!path_1.isAbsolute(item))
                return path_1.join(cwd, item);
            return item;
        });
        const newOptions = Object.assign(Object.assign({}, options), { ignore: absolutePathIgnored });
        const errors = [];
        yield execteFuncInSubproject(func, cwd, newOptions, errors);
        handleErrors(errors);
    }
});
